/* 
 * Copyright (C) 2001 Sonal Bansal <sonal_b@yahoo.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package com.javapal.CellQuotes;

import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

/* This is the servlet which represents the frontend to the CellQuotes
 * solution. On receving a GET request, it sends the new user form.
 * On receiving a POST, it extracts the information and writes it to the 
 * transaction flat file
 */
public class CellQuotes extends HttpServlet
{
	private static final int iLockMaxRetry = 5;
	private static final int lockWait = Constants.SEC*5;

	File lockFile = new File(Constants.lockFile);
	File transactionFile = new File(Constants.transactionFile);

	/* Responds to GET request, by sending new user info form
	 */
	public void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException
	{
		// It simply dumps a local HTML file to the response
		// stream.
		response.setContentType("text/html");

		File htmlFile = new File(Constants.getHTMLFile);
		PrintWriter responseWriter = response.getWriter();
		BufferedReader fileReader = new BufferedReader(new FileReader(htmlFile));

		String line;
		while((line = fileReader.readLine()) != null)
		{
			responseWriter.println(line);
			responseWriter.flush();
		}
		fileReader.close();
		responseWriter.close();
	}

	/* Extracts user info from form and saves to transaction flat file
	 */
	public void doPost(HttpServletRequest request,HttpServletResponse response) throws ServletException,IOException
	{
		// This method assumes that the user's inputs
		// have already been validated by client-side
		// JavaScript.

		Profile prof = new Profile();

		prof.setParameter(Profile.EMAIL_ID,request.getParameter("mailid"));
		prof.setParameter(Profile.PROVIDER,request.getParameter("provider"));
		prof.setParameter(Profile.NUMBER,request.getParameter("number"));
		prof.setParameter(Profile.FREQUENCY,request.getParameter("freq"));
		prof.setParameter(Profile.QUOTES,request.getParameter("quotes"));
		prof.setParameter(Profile.SYMBOLS,formatSymbols(request.getParameter("symbols")));
		prof.setParameter(Profile.NEWS,request.getParameter("news"));

		// Send an acknowledgement to the user
		response.setContentType("text/html");

		PrintWriter pw = response.getWriter();

		pw.println("<html><head><title> CellQuotes Wireless Web Service </title></head>");
		pw.println("<body><p>The following information were received :--<br>");
		pw.println("E-Mail ID : " + prof.getParameter(Profile.EMAIL_ID) + "<br>");
		pw.println("Service Provider : " + prof.getParameter(Profile.PROVIDER) + "<br>");
		pw.println("Number : " + prof.getParameter(Profile.NUMBER) + "<br>");
		pw.println("Frequency : " + prof.getParameter(Profile.FREQUENCY) + "<br>");
		if(prof.getQuotesStatus() == true)
		{
			pw.println("You want to receive Stock Quotes for the symbols - " + prof.getParameter(Profile.SYMBOLS) + "<br>");
		}
		if(prof.getNewsStatus() == true)
		{
			pw.println("You want to receive News HeadLines. <br>");
		}
		
		pw.println("</p><p>\nThat's All Folks.</p>");
		pw.println("</body></html>");
		pw.flush();
		pw.close();

		// Now write the Profile to the transaction flat file
		writeProfile(prof);
	}

	/* This method wraps the control logic to write a Profile to the 
	 * transaction flat file
	 */
	private void writeProfile(Profile p) throws IOException
	{
		// We need to synchronize on some common object so that
		// multiple servlet threads don't corrupt the flat file
		synchronized(Constants.syncObject)
		{
			// check if lockFile exists
			int lockCount = 0;
			while(++lockCount <= iLockMaxRetry)
			{
				if(lockFile.exists())
				{
					try
					{
						Thread.sleep(lockWait);
					}
					catch(InterruptedException ie)
					{
					}
				}
				else
					break;
			}

			if(lockCount > iLockMaxRetry)
			{
				System.out.println("Could not gain access to transaction file.");
				System.out.println("Profile could not be written.");
				return;
			}
			else
			{
				// We have access to transaction file, so create lockFile and add profiles
				FileOutputStream fos = new FileOutputStream(lockFile);
				fos.write(0);
				fos.close();

				ProfileWriter pw = new ProfileWriter(transactionFile);
				pw.writeProfile(p);
				pw.close();

				lockFile.delete();
			}
		}
	}

	/* This method standardizes the way stock symbols are represented.
	 * ie, comma separated or white space separated
	 */
	static String formatSymbols(String symb)
	{
		String symbols = new String();
		StringTokenizer st = new StringTokenizer(symb," ,;");
		while(st.hasMoreTokens())
		{
			symbols += st.nextToken();
			if(st.countTokens() != 0)
				symbols += " ";
		}
		return(symbols);
	}
}
