/* 
 * Copyright (C) 2001 Sonal Bansal <sonal_b@yahoo.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package com.javapal.CellQuotes;

import java.io.*;
import java.util.*;
import java.net.*;

/* This is the actual workhorse of the CellQuotes solution.
 * It contains the timing logic. It wakes up every hour, 
 * updates profile store, and dispatches appropriate information 
 * to "eligible" users.
 */
public class TimeWorker
{
	static int currentHour = 0;

	static final int iLockMaxRetry = 5;
	static final long lockWait = Constants.SEC*5;

	private static long waitTime = Constants.HOUR*1;

	/* This is so that it cannot be instantiated
	 */
	private TimeWorker()
	{
	}

	/* This is the nerve-center of the solution
	 */
	public static void main(String[] args)
	{
		File transactionFile = new File(Constants.transactionFile);
		File lockFile = new File(Constants.lockFile);
		File profilesFile = new File(Constants.profilesFile);

		try
		{
			// we want the application to run continuously, so enter an infinite loop
			while(true)
			{
				// check if Transaction file exists
				// if yes, then new profiles must be merged with main store
				if(transactionFile.exists())
				{
					// check if lockFile exists
					int lockCount = 0;
					while(++lockCount <= iLockMaxRetry)
					{
						if(lockFile.exists())
						{
							try
							{
								Thread.sleep(lockWait);
							}
							catch(InterruptedException ie)
							{
							}
						}
						else
							break;
					}

					if(lockCount > iLockMaxRetry)
					{
						System.out.println("Could not gain access to transaction file, continuing with processing.");
					}
					else
					{
						// We have access to transaction file, so create lockFile and merge profiles
						FileOutputStream fos = new FileOutputStream(lockFile);
						fos.write(0);
						fos.close();

						ProfileReader pr = new ProfileReader(transactionFile);
						ProfileWriter pw = new ProfileWriter(profilesFile);
						Profile p;

						while(pr.hasMoreProfiles())
						{
							p = pr.nextProfile();
							pw.writeProfile(p);
						}

						pr.close();
						pw.close();

						transactionFile.delete();
						lockFile.delete();
					}
				}

				// Main store is now up-to-date, go ahead with processing
				ProfileReader pr = new ProfileReader(profilesFile);
				Profile p;
				while(pr.hasMoreProfiles())
				{
					p = pr.nextProfile();
					// check if this user should be sent quotes
					if(isEligible(p, currentHour))
					{
						doDispatch(p);
					}
				}
				pr.close();

				currentHour++;
				if(currentHour > 24)
				{
					currentHour = 1;
				}

				// All Profiles have been dealt with
				// Now sleep for an hour, and get back to work
				try
				{
					Thread.sleep(waitTime);
				}
				catch(InterruptedException ie)
				{
				}
			}
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}

	/* This method takes care of deciding which services 
	 * (here, stocks and news headlines) should be sent
	 * to the user.
	 */
	private static void doDispatch(Profile prof) throws Exception
	{
		if(prof.getQuotesStatus() == true)
		{
			sendQuotes(prof);
		}

		if(prof.getNewsStatus() == true)
		{
			sendNews(prof);
		}
	}

	/* This method sends the stock quotes to the user */
	private static void sendQuotes(Profile prof) throws Exception
	{
		File tempFile = getQuotes(prof.getParameter(prof.SYMBOLS));

		BufferedReader quotesFile = new BufferedReader(new FileReader(tempFile));
		StringTokenizer st;
		int iCurField;
		String token;
		String line;
		String message = new String();
		String CRLF = "\r\n";
		while((line = quotesFile.readLine()) != null)
		{
			st = new StringTokenizer(line,"\",");
			iCurField = 1;
			while(st.hasMoreTokens() == true)
			{
				token = st.nextToken();
				if(iCurField == 2)
				{
					message += token + CRLF;
					break;
				}
				message += token + " ";
				iCurField++;
			}
		}
		quotesFile.close();
		tempFile.delete();
		sendMessage(prof.getParameter(prof.PROVIDER),message,prof.getParameter(prof.NUMBER));
	}

	/* This method sends news headlines to the user.
	 */
	private static void sendNews(Profile prof)
	{
		// We haven't implemented this ...
		// This is just to show how multiple services can be 
		// provided to the user
	}


	/* This method sends a text message to the specified user.
	 * It wraps the logic for retrieving appropriate CellProvider
	 * implementation.
	 */
	private static void sendMessage(String providerID,String message,String number) throws Exception
	{
		CellProviderSelector.setRegistryFile(Constants.providerRegistryFile);
		CellProvider provider = CellProviderSelector.getProvider(providerID);

		if(provider != null)
			provider.sendMessage(number,message);
	}

	/* This method retrieves the quotes for specified stock symbols 
	 * from http://quote.yahoo.com
	 */	
	private static File getQuotes(String strSymbols) throws Exception
	{
		String csvFileUrl = "http://quote.yahoo.com/d/quotes.csv?s=";
		csvFileUrl += URLEncoder.encode(strSymbols);
		csvFileUrl += "&f=sl1d1t1c1ohgv&e=.csv";

		URL url = new URL(csvFileUrl); 
		
		HttpURLConnection conn = (HttpURLConnection) url.openConnection();

		InputStream is = conn.getInputStream();
		File tempQuotesFile = new File(Constants.tempQuotesFile);

		FileOutputStream fos = new FileOutputStream(tempQuotesFile);

		int oneChar;

		while((oneChar = is.read()) != -1)
		{
			fos.write(oneChar);
		}
		fos.close();
		is.close();
		return(tempQuotesFile);
	}

	/* Given the current hour, this method determines 
	 * if this user should be sent the quotes
	 */
	private static boolean isEligible(Profile prof,int currentHour)
	{
		int freq;
		freq = prof.getFrequency();

		if(currentHour % freq == 0)
		{
			return(true);
		}

		return(false);
	}
}
